"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processBody = void 0;
/* eslint-disable @typescript-eslint/no-use-before-define */
const lodash_1 = require("lodash");
const template_processor_1 = require("./template-processor");
const collection_processor_1 = require("./collection-processor");
const container_processor_1 = require("./container-processor");
const cache = {};
async function processBody(config, context) {
    const cacheKey = config.method + context.url;
    if (config.cache && !(0, lodash_1.isNil)(cache[cacheKey])) {
        return cache[cacheKey];
    }
    const body = await containerize(config, context, await buildBody(config, context));
    cache[cacheKey] = body;
    return body;
}
exports.processBody = processBody;
async function buildBody(config, context) {
    if ((0, collection_processor_1.isCollection)(config)) {
        return (0, collection_processor_1.processCollection)(config, context);
    }
    return (0, template_processor_1.processTemplate)(config.template, context);
}
async function containerize(config, context, generatedBody) {
    if ((0, lodash_1.isNil)(config.container)) {
        return generatedBody;
    }
    return (0, container_processor_1.processContainer)(config.container, context, generatedBody);
}
//# sourceMappingURL=response-body-processor.js.map