export declare type RouteConfig = CollectionRouteConfig | SimpleRouteConfig;
export declare type SimpleRouteConfig = {
    method: string;
    path: string;
    cache?: boolean;
    container?: Container;
    status?: StatusFn;
    template: Template;
};
export declare type CollectionRouteConfig = SimpleRouteConfig & {
    collection: boolean;
    collectionSize: number | CollectionSizeFn<unknown>;
};
export declare type RequestParams = Record<string, string>;
export declare type RequestQuery = Record<string, string | string[]>;
export declare type RequestPayload<T> = T;
export declare type RequestContext<T> = {
    params?: RequestParams;
    query?: RequestQuery;
    payload?: RequestPayload<T>;
    url: string;
};
export declare type StatusFn = (ctx: RequestContext<unknown>) => number;
export declare type CollectionSizeFn<T> = (ctx: RequestContext<T>) => number;
export declare type ContainerLeafValue = string | boolean | number | null | object;
export declare type ContainerFunction = (context: RequestContext<unknown>, templateGenerated: unknown) => ContainerLeafValue;
export declare type ContainerObjectValue = ContainerLeafValue | ContainerFunction;
export declare type ContainerObject = {
    [key in string]: ContainerObjectValue;
};
export declare type Container = ContainerLeafValue | ContainerObject | ContainerFunction;
export declare type TemplateLeafValue = string | boolean | number | null | object;
export declare type TemplateFunction = (context: RequestContext<unknown>) => TemplateLeafValue;
export declare type TemplateObjectValue = TemplateLeafValue | TemplateFunction;
export declare type TemplateObject = {
    [key in string]: TemplateObjectValue;
};
export declare type Template = TemplateLeafValue | TemplateObject | TemplateFunction;
