"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StubApiServer = void 0;
const hapi_1 = require("@hapi/hapi");
const boom_1 = require("@hapi/boom");
const route_builder_1 = require("./route-builder");
const UNMATCHED_ROUTE = {
    method: '*',
    path: '/{any*}',
    handler: function () {
        throw (0, boom_1.notImplemented)();
    },
};
class StubApiServer {
    constructor(options) {
        var _a;
        this.options = options;
        this.app = new hapi_1.Server({ port: (_a = this.options) === null || _a === void 0 ? void 0 : _a.port });
        this.app.route(UNMATCHED_ROUTE);
    }
    useRoutes(routes) {
        (0, route_builder_1.buildFromRouteConfig)(routes).map(r => this.addRoute(r));
        return this;
    }
    async useRoutesFromDir(path) {
        (await (0, route_builder_1.buildFromDirectory)(path)).map(r => this.addRoute(r));
        return this;
    }
    async start() {
        await this.app.start();
    }
    stop() {
        return this.app.stop();
    }
    listeningUrl() {
        return `http://localhost:${this.app.info.port}`;
    }
    addRoute(r) {
        this.app.route({
            method: r.method,
            path: r.path,
            handler: async (request, h) => {
                const res = await r.handler({
                    params: request.params,
                    query: request.query,
                    payload: request.payload,
                    url: request.url.toString(),
                });
                return h.response(res.body).code(res.status);
            },
        });
    }
}
exports.StubApiServer = StubApiServer;
//# sourceMappingURL=stub-api-server.js.map